@@ -36,12 +36,16 @@ __license__ = "Apache License, Version 2.0"
 
 from . import admin
 from . import captcha
+from . import contentful
 from . import csfr
 from . import opbeat
+from . import prismic
 from . import sematext
 
 from .admin import AdminPart
 from .captcha import CaptchaPart
+from .contentful import Contentful
 from .csfr import CSFRPart, csfr_protect, csfr_ensure
 from .opbeat import OpbeatPart
+from .prismic import Prismic
 from .sematext import SematextHandler, SematextPart
