@@ -38,6 +38,7 @@ __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
 import csv
+import json
 import time
 import random
 import string
@@ -237,6 +238,13 @@ class Base(appier.Model):
     def is_snapshot(cls):
         return False
 
+    @classmethod
+    def secret_g(cls):
+        token = "".join(random.choice(RANDOM_RANGE) for _index in range(32))
+        token_bytes = appier.legacy.bytes(token)
+        url_sha1 = hashlib.sha1(token_bytes)
+        return url_sha1.hexdigest()
+
     @classmethod
     def build_index_g(cls, *args, **kwargs):
         models = cls.find(*args, **kwargs)
@@ -332,6 +340,43 @@ class Base(appier.Model):
         cls,
         file,
         callback,
+        callback_header = None,
+        strict = False,
+        named = False,
+        delimiter = ",",
+        quotechar = "\"",
+        quoting = csv.QUOTE_MINIMAL,
+        encoding = "utf-8"
+    ):
+        is_unicode = appier.legacy.PYTHON_3
+        csv_reader = cls._csv_read(
+            file,
+            strict = strict,
+            named = named,
+            delimiter = delimiter,
+            quotechar = quotechar,
+            quoting = quoting,
+            encoding = encoding
+        )
+        args, _varargs, kwargs = appier.legacy.getargspec(callback)[:3]
+        has_header = True if "header" in args or kwargs else False
+        has_map = True if "map" in args or kwargs else False
+        header = next(csv_reader)
+        if not is_unicode: header = [value.decode(encoding) for value in header]
+        if callback_header: callback_header(header)
+        if named: tuple_t = collections.namedtuple("csv_tuple", header)
+        for line in csv_reader:
+            kwargs = dict()
+            if not is_unicode: line = [value.decode(encoding) for value in line]
+            if named: line = tuple_t(*line)
+            if has_header: kwargs["header"] = header
+            if has_map: kwargs["map"] = dict(zip(header, line))
+            callback(line, **kwargs)
+
+    @classmethod
+    def _csv_read(
+        cls,
+        file,
         strict = False,
         named = False,
         delimiter = ",",
@@ -344,11 +389,8 @@ class Base(appier.Model):
         is_csv = mime_type in ("text/csv", "application/vnd.ms-excel")
         if not is_csv and strict:
             raise appier.OperationalError(
-                message = "Invalid mime type '%s'" % mime_type
+                message = "Invalid MIME type '%s'" % mime_type
             )
-        args, _varargs, kwargs = appier.legacy.getargspec(callback)[:3]
-        has_header = True if "header" in args or kwargs else False
-        has_map = True if "map" in args or kwargs else False
         if is_unicode:
             data = data.decode(encoding)
             buffer = appier.legacy.StringIO(data)
@@ -360,30 +402,55 @@ class Base(appier.Model):
             quotechar = quotechar,
             quoting = quoting
         )
-        header = next(csv_reader)
-        if named: tuple_t = collections.namedtuple("csv_tuple", header)
-        for line in csv_reader:
-            kwargs = dict()
-            if not is_unicode: line = [value.decode(encoding) for value in line]
-            if named: line = tuple_t(*line)
-            if has_header: kwargs["header"] = header
-            if has_map: kwargs["map"] = dict(zip(header, line))
-            callback(line, **kwargs)
+        return csv_reader
 
     @classmethod
-    def _inlinify(cls, data, engine = None):
+    def _json_import(
+        cls,
+        file,
+        callback,
+        callback_header = None,
+        strict = False,
+        encoding = "utf-8"
+    ):
+        json_data = cls._json_read(file, strict = strict, encoding = encoding)
+        header = appier.legacy.keys(json_data[0]) if json_data else []
+        if callback_header: callback_header(header)
+        for item in json_data: callback(item)
+
+    @classmethod
+    def _json_read(
+        cls,
+        file,
+        strict = False,
+        encoding = "utf-8"
+    ):
+        _file_name, mime_type, data = file
+        is_json = mime_type in ("text/json", "application/json")
+        if not is_json and strict:
+            raise appier.OperationalError(
+                message = "Invalid MIME type '%s'" % mime_type
+            )
+        data = data.decode(encoding)
+        json_data = json.loads(data)
+        return json_data
+
+    @classmethod
+    def _inlinify(cls, data, engine = None, *args, **kwargs):
         engine = engine or appier.conf("INLINER_ENGINE", None)
         if not engine: return data
         method = getattr(cls, "_inlinify_" + engine)
-        return method(data)
+        return method(data, *args, **kwargs)
 
     @classmethod
-    def _inlinify_premailer(cls, data):
+    def _inlinify_premailer(cls, data, *args, **kwargs):
         premailer = appier.import_pip("premailer")
-        return premailer.transform(data)
+        keep_style_tags = kwargs.get("keep_style_tags", True)
+        inliner = premailer.Premailer(data, keep_style_tags = keep_style_tags)
+        return inliner.transform(data)
 
     @classmethod
-    def _inlinify_toronado(cls, data):
+    def _inlinify_toronado(cls, data, *args, **kwargs):
         toronado = appier.import_pip("toronado")
         return toronado.from_string(data)
 
@@ -534,18 +601,25 @@ class Base(appier.Model):
     def touch_s(self):
         self.save()
 
+    def update_meta_s(self, meta = None, **kwargs):
+        meta = kwargs if meta == None else meta
+        appier.verify(isinstance(meta, dict))
+        if not self.meta: self.meta = meta
+        else: self.meta.update(meta)
+        self.save()
+
     def to_locale(self, *args, **kwargs):
         return self.owner.to_locale(*args, **kwargs)
 
     def send_email(self, *args, **kwargs):
         cls = self.__class__
-        return cls.send_email_g(self.owner, *args, **kwargs)
+        owner = kwargs.pop("owner", self.owner)
+        kwargs["own"] = kwargs.pop("own", self)
+        return cls.send_email_g(owner, *args, **kwargs)
 
     def secret(self):
-        token = "".join(random.choice(RANDOM_RANGE) for _index in range(32))
-        token_bytes = appier.legacy.bytes(token)
-        url_sha1 = hashlib.sha1(token_bytes)
-        return url_sha1.hexdigest()
+        cls = self.__class__
+        return cls.secret_g()
 
     @appier.operation(name = "Enable")
     def op_enable_s(self):
