@@ -124,6 +124,10 @@ class Search(base.Base):
         indexes = cls.find_indexes(target_id, target_cls, build = False)
         for index in indexes: index.delete(*args, **kwargs)
 
+    @classmethod
+    def _plural(cls):
+        return "Searches"
+
     @classmethod
     def _build(cls, model, map):
         super(Search, cls)._build(model, map)
