@@ -45,6 +45,8 @@ import tempfile
 
 import appier
 
+from appier_extras import base
+from appier_extras import utils
 from appier_extras.parts.admin import models
 from appier_extras.parts.admin import social
 
@@ -74,11 +76,34 @@ class AdminPart(
     ):
         appier.Part.__init__(self, *args, **kwargs)
         self.account_c = account_c
-        self.layout = appier.conf("ADMIN_LAYOUT", "fluid")
-        self.theme = appier.conf("ADMIN_THEME", "flat")
-        self.style = appier.conf("ADMIN_STYLE", "")
-        self.libs = appier.conf("ADMIN_LIBS", "current")
-        self.social_libs = appier.conf("ADMIN_SOCIAL_LIBS", [], cast = list)
+        self.role_c = role_c
+        self.layout = kwargs.get("layout", "fluid")
+        self.theme = kwargs.get("theme", "flat")
+        self.style = kwargs.get("style", "")
+        self.libs = kwargs.get("libs", "current")
+        self.social_libs = kwargs.get("libs", [])
+        self.available = kwargs.get("available", True)
+        self.open = kwargs.get("open", False)
+        self.oauth = kwargs.get("oauth", True)
+        self.avatar_default = kwargs.get("avatar_default", False)
+        self.layout = appier.conf("ADMIN_LAYOUT", self.layout)
+        self.theme = appier.conf("ADMIN_THEME", self.theme)
+        self.style = appier.conf("ADMIN_STYLE", self.style)
+        self.libs = appier.conf("ADMIN_LIBS", self.libs)
+        self.social_libs = appier.conf("ADMIN_SOCIAL_LIBS", self.social_libs, cast = list)
+        self._last_login = None
+        self._login_count = 0
+
+    def version(self):
+        return base.VERSION
+
+    def info(self):
+        info = appier.Part.info(self)
+        info.update(
+            last_login = self._last_login_s,
+            login_count = self._login_count
+        )
+        return info
 
     def load(self):
         appier.Part.load(self)
@@ -97,6 +122,7 @@ class AdminPart(
         self.owner.login_redirect = "admin.index"
         self.owner.logout_redirect = "admin.login"
         self.owner.admin_account = self.account_c
+        self.owner.admin_role = self.role_c
         self.owner.admin_login_redirect = "admin.index"
         self.owner.admin_logout_redirect = "admin.login"
         self.owner.admin_facebook_scope = ("email",)
@@ -104,14 +130,17 @@ class AdminPart(
         self.owner.admin_google_scope = ("email",)
         self.owner.admin_live_scope = ("wl.basic", "wl.emails")
 
+        self.owner.admin_available = appier.conf(
+            "ADMIN_AVAILABLE", self.available, cast = bool
+        )
         self.owner.admin_open = appier.conf(
-            "ADMIN_OPEN", False, cast = bool
+            "ADMIN_OPEN", self.open, cast = bool
         )
         self.owner.admin_oauth = appier.conf(
-            "ADMIN_OAUTH", True, cast = bool
+            "ADMIN_OAUTH", self.oauth, cast = bool
         )
         self.owner.admin_avatar_default = appier.conf(
-            "ADMIN_AVATAR_DEFAULT", False, cast = bool
+            "ADMIN_AVATAR_DEFAULT", self.avatar_default, cast = bool
         )
 
         self.owner.lib_loaders["appier_extras"] = self._appier_extras_loader
@@ -122,18 +151,31 @@ class AdminPart(
         self.owner.lib_loaders["jinja2"] = self._jinja2_loader
         self.owner.lib_loaders["ssl"] = self._ssl_loader
 
+        self.owner.add_filter(self.markdown_jinja, "markdown")
+
         if self.owner.allow_headers: self.owner.allow_headers += ", X-Secret-Key"
 
         self.logger.debug("Generating admin interfaces ...")
         for model_c in self.models_r:
             if not model_c.is_attached(): continue
             if not model_c.is_concrete(): continue
+            if not model_c.is_child(models.Base): continue
             self.logger.debug(model_c)
 
         for social_lib in self.social_libs:
             method = getattr(self, "ensure_" + social_lib)
             method()
 
+        self.load_settings()
+
+        self.account_c.bind_g("touch_login", self._on_touch_login)
+
+    def unload(self):
+        appier.Part.unload(self)
+
+        self.account_c.unbind_g("touch_login", self._on_touch_login)
+        self.owner.remove_filter("markdown")
+
     def routes(self):
         return [
             (("GET",), "/admin", self.index),
@@ -153,6 +195,7 @@ class AdminPart(
             (("GET",), "/admin/routes", self.list_routes),
             (("GET",), "/admin/configs", self.list_configs),
             (("GET",), "/admin/parts", self.list_parts),
+            (("GET",), "/admin/parts/<str:name>", self.show_part),
             (("GET",), "/admin/libraries", self.list_libraries),
             (("GET",), "/admin/oauth/authorize", self.oauth_authorize),
             (("POST",), "/admin/oauth/authorize", self.do_oauth_authorize),
@@ -260,6 +303,24 @@ class AdminPart(
             lines = lines
         )
 
+    def load_settings(self):
+        settings = self.owner.get_preference(self.name() + ":settings")
+        if not settings: return
+        for name in self.settings_l:
+            current = getattr(self, name)
+            value = settings.get(name, current)
+            setattr(self, name, value)
+
+    def dump_settings(self):
+        settings = dict()
+        for name in self.settings_l:
+            value = getattr(self, name)
+            settings[name] = value
+        self.owner.set_preference(self.name() + ":settings", settings)
+
+    def flush_settings(self):
+        self.dump_settings()
+
     def index(self):
         return self.list_models()
 
@@ -275,6 +336,12 @@ class AdminPart(
         )
 
     def login(self):
+        # verifies if the current administration interface is
+        # available and if that's not the cases raises an error
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
+
         # retrieves the various fields that are going to be
         # used for the validation of the user under the current
         # authentication/authorization process
@@ -296,7 +363,7 @@ class AdminPart(
         # values to correctly authenticate the user
         account._set_session()
 
-        # redirects the current operation to the next url or in
+        # redirects the current operation to the next URL or in
         # alternative to the root index of the administration
         return self.redirect(
             next or self.url_for(self.owner.admin_login_redirect)
@@ -392,6 +459,9 @@ class AdminPart(
         )
 
     def new_account(self):
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
         if not self.owner.admin_open: raise appier.SecurityError(
             message = "Signup not allowed"
         )
@@ -402,6 +472,9 @@ class AdminPart(
         )
 
     def create_account(self):
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
         if not self.owner.admin_open: raise appier.SecurityError(
             message = "Signup not allowed"
         )
@@ -424,6 +497,7 @@ class AdminPart(
             )
         )
 
+    @appier.ensure()
     def me_account(self):
         account_c = self._get_cls(self.account_c)
         account = account_c.from_session(meta = True)
@@ -432,6 +506,7 @@ class AdminPart(
             account = account
         )
 
+    @appier.ensure("admin.accounts")
     def show_account(self, username):
         account_c = self._get_cls(self.account_c)
         account = account_c.get(
@@ -443,6 +518,7 @@ class AdminPart(
             account = account
         )
 
+    @appier.ensure("admin.accounts")
     def mail_account(self, username):
         raise appier.NotImplementedError()
 
@@ -557,6 +633,20 @@ class AdminPart(
             parts = parts
         )
 
+    @appier.ensure(token = "admin.status")
+    def show_part(self, name):
+        part = self.owner.get_part(name)
+        info = part.info()
+        info_l = appier.legacy.keys(info)
+        info_l.sort()
+        return self.template(
+            "part.html.tpl",
+            section = "status",
+            part = part,
+            info = info,
+            info_l = info_l
+        )
+
     @appier.ensure(token = "admin.status")
     def list_libraries(self):
         libraries = self.owner.get_libraries()
@@ -719,6 +809,12 @@ class AdminPart(
         )
 
     def oauth_login(self):
+        # verifies if the current administration interface is
+        # available and if that's not the cases raises an error
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
+
         # verifies if the oauth system is allowed if that's not
         # the case raises an exception indicating so
         if not self.owner.admin_oauth: raise appier.SecurityError(
@@ -741,7 +837,7 @@ class AdminPart(
         # requests to refer to the proper session
         sid = self.session.sid
 
-        # redirects the current operation to the next url or in
+        # redirects the current operation to the next URL or in
         # alternative to the root index of the administration
         return dict(
             sid = sid,
@@ -754,7 +850,7 @@ class AdminPart(
     def build_index(self):
         empty = self.field("empty", True, cast = bool)
         if empty: models.Search.delete_c()
-        for model in self._attached(self.models_r):
+        for model in self._administrable(self.models_r):
             model.build_index_g()
         return self.redirect(
             self.url_for(
@@ -765,7 +861,7 @@ class AdminPart(
 
     @appier.ensure(token = "admin")
     def build_index_db(self):
-        for model in self._attached(self.models_r):
+        for model in self._administrable(self.models_r):
             model._destroy_indexes()
             model._build_indexes()
         return self.redirect(
@@ -1005,7 +1101,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def show_model(self, model):
         appier.ensure_login(self, token = "admin.models." + model)
-        model = self.get_model(model)
+        model = self.get_model(model, raise_e = True)
         model.assert_is_concrete_g()
         object = appier.get_object(
             alias = True,
@@ -1065,6 +1161,10 @@ class AdminPart(
         # are going to be applied to the link operations
         parameters = self.get_fields("parameters", [])
 
+        # retrieves the reference to the view that is going to be used
+        # to filter the entities for which to generate the link (if any)
+        view = self.field("view", None)
+
         # tries to get the context reference that is going to be "passed"
         # to the underlying link operation if required
         context = self.field("context", None)
@@ -1094,9 +1194,11 @@ class AdminPart(
         # in case there's some (eg: context)
         kwargs = dict()
 
-        # creates the dictionary that is going to hold the named
-        # arguments for the URL generation
-        if context: kwargs["context"] = context
+        # applies both the view and the context to the keyword based
+        # dictionary to be passed to the link (generation) method, they
+        # are going to constrain the domain for the link execution
+        if view: kwargs["view"] = view
+        if context: kwargs["context"] = "_id:in:" + ";".join(context)
 
         # defines the default result value as an invalid value,
         # this will ensure an error in the redirection process
@@ -1124,6 +1226,7 @@ class AdminPart(
         # complete set of casting operations for each of them
         parameters = self.get_fields("parameters", [])
         next = self.field("next")
+        view = self.field("view", None)
         ids = self.field("ids", "")
         ids = ids.split(",")
         ids = [self.get_adapter().object_id(_id) for _id in ids if _id]
@@ -1134,6 +1237,10 @@ class AdminPart(
         model = self.get_model(model)
         definition = model.operation(operation)
 
+        # uses the definition of the operation to retrieve its descriptive
+        # name to be used in the message to be sent to the end-user
+        operation_s = definition.get("name", operation)
+
         # using the retrieved information determines if the operation
         # is considered to be a factory one (generates entity) and
         # then casts the provided parameters according to metadata
@@ -1147,6 +1254,7 @@ class AdminPart(
         # if this is a global operation classes are used instead
         if ids: kwargs = dict(_id = {"$in" : ids})
         else: kwargs = dict()
+        kwargs = self._apply_view(view, kwargs = kwargs, cls = model)
         if is_global: entities = (model,)
         else: entities = model.find_v(**kwargs)
 
@@ -1182,6 +1290,7 @@ class AdminPart(
             return self.redirect(
                 self.url_for(
                     "admin.show_entity",
+                    message = "Operation '%s' completed with success" % operation_s,
                     model = model_name,
                     _id = model_id
                 )
@@ -1190,7 +1299,12 @@ class AdminPart(
         # runs the default redirection process that displays the model
         # page for the model used as target for the operation
         return self.redirect(
-            next or self.url_for("admin.show_model", model = model._under())
+            next or self.url_for(
+                "admin.show_model",
+                message = "Operation '%s' completed with success" % operation_s,
+                model = model._under()
+            ),
+            message = "Operation '%s' completed with success" % operation_s
         )
 
     @appier.ensure(token = "admin")
@@ -1201,6 +1315,9 @@ class AdminPart(
         _id = self.field("id", None)
         is_global = False if _id else True
 
+        model_s = model
+        parameters_s = ",".join(parameters)
+
         model = self.get_model(model)
         model.assert_is_concrete_g()
 
@@ -1227,6 +1344,10 @@ class AdminPart(
         page = result["page"]
         names = result.get("names", None)
 
+        if _id: view_s = "%s:%s.%s" % (model_s, _id, view)
+        else: view_s = "%s.%s" % (model_s, view)
+        if parameters_s: view_s += "(%s)" % parameters_s
+
         return self.template(
             "views/show.html.tpl",
             section = "models",
@@ -1237,6 +1358,7 @@ class AdminPart(
             page = page,
             definition = definition,
             names = names,
+            view = view_s,
             is_global = is_global
         )
 
@@ -1272,7 +1394,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def show_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
-        model = self.get_model(model)
+        model = self.get_model(model, raise_e = True)
         entity = model.get_v(
             rules = False,
             meta = True,
@@ -1294,7 +1416,7 @@ class AdminPart(
         eager_l = self.field("eager_l", True, cast = bool)
         rules = self.field("rules", False, cast = bool)
         meta = self.field("meta", False, cast = bool)
-        model = self.get_model(model)
+        model = self.get_model(model, raise_e = True)
         entity = model.get_v(
             eager_l = eager_l,
             rules = rules,
@@ -1307,7 +1429,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def edit_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
-        model = self.get_model(model)
+        model = self.get_model(model, raise_e = True)
         entity = model.get_v(
             rules = False,
             meta = True,
@@ -1324,7 +1446,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def update_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
-        model = self.get_model(model)
+        model = self.get_model(model, raise_e = True)
         entity = model.get_v(
             rules = False,
             meta = True,
@@ -1352,7 +1474,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def delete_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
-        model = self.get_model(model)
+        model = self.get_model(model, raise_e = True)
         entity = model.get_v(_id = self.get_adapter().object_id(_id))
         entity.delete()
         return self.redirect(
@@ -1400,6 +1522,9 @@ class AdminPart(
         )
 
     def oauth_facebook(self):
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
         code = self.field("code")
         state = self.field("state")
         context, next = state.split(":", 1)
@@ -1454,6 +1579,9 @@ class AdminPart(
         )
 
     def oauth_github(self):
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
         code = self.field("code")
         state = self.field("state")
         context, next = state.split(":", 1)
@@ -1515,6 +1643,9 @@ class AdminPart(
         )
 
     def oauth_google(self):
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
         code = self.field("code")
         state = self.field("state")
         context, next = state.split(":", 1)
@@ -1577,6 +1708,9 @@ class AdminPart(
         )
 
     def oauth_live(self):
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
         code = self.field("code")
         state = self.field("state")
         context, next = state.split(":", 1)
@@ -1629,6 +1763,9 @@ class AdminPart(
         )
 
     def oauth_twitter(self):
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
         oauth_verifier = self.field("oauth_verifier")
         state = self.field("state")
         context, next = state.split(":", 1)
@@ -1665,6 +1802,12 @@ class AdminPart(
         return dict(time = time.time())
 
     def login_api(self):
+        # verifies if the current administration interface is
+        # available and if that's not the cases raises an error
+        if not self.owner.admin_available: raise appier.SecurityError(
+            message = "Administration not available"
+        )
+
         # retrieves the various fields that are going to be
         # used for the validation of the user under the current
         # authentication/authorization process
@@ -1681,7 +1824,7 @@ class AdminPart(
         # requests to refer to the proper session
         sid = self.session.sid
 
-        # redirects the current operation to the next url or in
+        # redirects the current operation to the next URL or in
         # alternative to the root index of the administration
         return dict(
             sid = sid,
@@ -1690,7 +1833,7 @@ class AdminPart(
             tokens = account.tokens()
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure()
     def me_account_api(self):
         account_c = self._get_cls(self.account_c)
         account = account_c.from_session(map = True)
@@ -1708,6 +1851,38 @@ class AdminPart(
     def linked(self):
         return models.Settings.linked_apis()
 
+    def markdown_jinja(self, value, *args, **kwargs):
+        result = self.markdown(value, *args, **kwargs)
+        return self.owner.escape_template(result)
+
+    def markdown(
+        self,
+        value,
+        anchors = False,
+        blank = True,
+        encoding = "utf-8"
+    ):
+        value = utils.MarkdownHTML.process_str(
+            value,
+            options = dict(
+                anchors = anchors,
+                blank = blank
+            ),
+            encoding = encoding
+        )
+        return appier.legacy.str(
+            value,
+            encoding = encoding,
+            force = True
+        )
+
+    @property
+    def settings_l(self):
+        return (
+            "_last_login",
+            "_login_count"
+        )
+
     def _counters(self):
         adapter = self.get_adapter()
         collection = adapter.collection("counters")
@@ -1761,15 +1936,36 @@ class AdminPart(
     def _attached(self, models):
         return [model for model in models if model.is_attached()]
 
+    def _visible(self, models):
+        return [model for model in models if model.is_visible()]
+
     def _concrete(self, models):
         models = self._attached(models)
         return [model for model in models if model.is_concrete()]
 
-    def _available(self, models):
-        models = self._concrete(models)
+    def _administrable(self, _models, parent = None):
+        parent = parent or models.Base
+        _models = self._concrete(_models)
+        return [model for model in _models if model.is_child(parent)]
+
+    def _accessible(self, models):
         return [model for model in models if\
              appier.check_login(self, "admin.models." + model._under())]
 
+    def _available(self, models):
+        models = self._administrable(models)
+        models = self._visible(models)
+        models = self._accessible(models)
+        return models
+
+    def _is_available(self, model, parent = None):
+        parent = parent or models.Base
+        if not model.is_child(parent): return False
+        if not model.is_concrete(): return False
+        if not model.is_visible(): return False
+        if not appier.check_login(self, "admin.models." + model._under()): return False
+        return True
+
     def _sort(self, object, model):
         if "sort" in object: return object
         order = model.order_name()
@@ -1813,10 +2009,10 @@ class AdminPart(
     def _to_meta(self, type):
         return appier.Model._to_meta(type)
 
-    def _get_cls(self, default = None):
+    def _get_cls(self, default = None, raise_e = True):
         cls = self.field("cls", None)
         if not cls: return default
-        return self.owner.get_model(cls)
+        return self.owner.get_model(cls, raise_e = raise_e)
 
     def _entity_urls(self, entity):
         model = entity.__class__
@@ -1833,3 +2029,101 @@ class AdminPart(
             _id = next._id
         ) if next else None
         return previous_url, next_url
+
+    def _find_view(self, cls, *args, **kwargs):
+        """
+        Runs a find operation taking into account the provided
+        view (as a field) constraining the domain.
+
+        This method is extremely useful for back-office operations
+        that use the dynamic view filtering.
+
+        :type cls: Class
+        :param cls: The (model) class that is going to be used in
+        the find operation.
+        :rtype: List
+        :return: The complete set of results from the data source
+        according to the provided parameters and filtered view.
+        """
+
+        view = self.field("view", None)
+        kwargs = self._apply_view(view, kwargs = kwargs, cls = cls)
+        return cls.find(*args, **kwargs)
+
+    def _apply_view(self, view, kwargs = None, cls = None):
+        """
+        Applies a certain view (with properly defined syntax) into
+        the provided arguments so that it's possible to run the
+        final result in a typical find operation.
+
+        Optionally one may provide the model class so that the view
+        is ensured to be associated with such class.
+
+        :type view: String
+        :param view: The string describing the view that should be
+        applied to the provided arguments.
+        :type kwargs: Dictionary
+        :param kwargs: The dictionary that contains the extra arguments
+        that are going to be affected by the view apply operation.
+        This object is going to be mutated.
+        :type cls: Class
+        :param cls: The class that if provided is going to be validated
+        against the view, meaning that only views that were designed to
+        returns entities of this class are going to pass validation.
+        :rtype: Dictionary
+        :return: The final dictionary with the view filter applied.
+        """
+
+        kwargs = dict() if kwargs == None else kwargs
+
+        if not view: return kwargs
+
+        model, view = view.split(".")
+
+        is_instance = ":" in model
+        if is_instance: model, id = model.split(":")
+
+        model = self.get_model(model, raise_e = True)
+        model.assert_is_concrete_g()
+
+        if is_instance: model = model.get(
+            _id = self.get_adapter().object_id(id)
+        )
+
+        start_param = view.find("(")
+        end_param = view.find(")")
+        is_valid = not start_param == -1 and not end_param == -1
+
+        if is_valid:
+            view, parameters = view[:start_param], view[start_param + 1:end_param]
+            parameters = [parameter.strip() for parameter in parameters.split(",")]
+        else:
+            parameters = []
+
+        definition = model.view(view)
+        parameters = definition.cast(parameters)
+        parameters_kw = definition.cast(parameters, keyword = True)
+        parameters_kw.update(kwargs)
+
+        method = getattr(model, view)
+        method_kw = appier.legacy.getargspec(method)[2]
+        result = method(**parameters_kw) if method_kw else method(*parameters, **kwargs)
+
+        appier.verify(not cls or result["model"] == cls)
+
+        kwargs = result["kwargs"]
+        return kwargs
+
+    def _on_touch_login(self, account):
+        self._last_login = time.time()
+        self._login_count += 1
+        self.flush_settings()
+
+    @property
+    def _last_login_s(self, format = "%Y-%m-%d %H:%M:%S UTC"):
+        if not self._last_login: return None
+        try:
+            date_time = datetime.datetime.utcfromtimestamp(self._last_login)
+            return date_time.strftime(format)
+        except TypeError:
+            return None
