@@ -42,6 +42,8 @@ import traceback
 
 import appier
 
+from appier_extras import base
+
 class OpbeatPart(appier.Part):
     """
     Modular part class that provides the system to handle Opbeat
@@ -53,6 +55,9 @@ class OpbeatPart(appier.Part):
     :see: http://opbeat.com
     """
 
+    def version(self):
+        return base.VERSION
+
     def load(self):
         appier.Part.load(self)
 
@@ -104,7 +109,7 @@ class OpbeatPart(appier.Part):
             http = dict(
                 url = self.url_for("location", absolute = True),
                 method = self.request.method,
-                data = self.request.get_data(),
+                data = self.request.get_encoded(),
                 query_string = self.request.query,
                 cookies = self.request.get_header("Cookie"),
                 headers = self.request.in_headers,
@@ -132,5 +137,5 @@ class OpbeatPart(appier.Part):
         if self._api: return self._api
         try: import opbeat
         except: return None
-        self._api = opbeat.Api()
+        self._api = opbeat.API()
         return self._api
