@@ -41,6 +41,8 @@ import logging
 
 import appier
 
+from appier_extras import base
+
 from . import handler
 
 class SematextPart(appier.Part):
@@ -54,6 +56,9 @@ class SematextPart(appier.Part):
     :see: http://sematext.com
     """
 
+    def version(self):
+        return base.VERSION
+
     def load(self):
         appier.Part.load(self)
 
@@ -84,5 +89,5 @@ class SematextPart(appier.Part):
         if self._api: return self._api
         try: sematext = appier.import_pip("sematext", package = "sematext_api")
         except: return None
-        self._api = sematext.Api()
+        self._api = sematext.API()
         return self._api
